#ifndef KADU_USERLIST_H
#define KADU_USERLIST_H

#include <qdns.h>
#include <qhostaddress.h>
#include <qobject.h>

#include "gadu.h"

class UserList;
class UserListElement;

/**
	Tumaczy adres IP na nazw domeny (DNS). Kiedy proces zostanie zakoczony,
	emituje odpowiedni sygna zawierajcy informacj o nazwie domeny.
	\class DnsHandler
	\brief Klasa tumaczca adres IP na nazw domeny.
**/
class DnsHandler : public QObject
{
	Q_OBJECT

	public:
		/**
			\fn DnsHandler(const QHostAddress &addr);
			Konstruktor wywoujcy zapytanie o domen dla danego adresu IP.
			\param addr adres IP
		**/
		DnsHandler(const QHostAddress &addr);
		~DnsHandler();

		static int counter; /*!< licznik obiektw tej klasy */

	private:
		QDns DnsResolver; /*!< obiekt zajmujcy si translacj adresu na nazw domeny */

	private slots:
		/**
			\fn void resultsReady()
			Funkcja wywoywana, gdy proces tumaczenia zosta zakoczony.
		**/
		void resultsReady();

	signals:
		/**
			\fn void result(const QString &hostname)
			Sygna emitowany, gdy proces tumaczenia zosta zakoczony.
			\param hostname nazwa domeny odpowiadajca adresowi IP o ktry pytano
		**/
		void result(const QString &hostname);
};

/**
	Typ opisujcy stan powiadamiania.
	\enum NotifyType
	\brief Typ opisujcy stan powiadamiania.
**/
enum NotifyType {
	GLOBAL = 0, /*!< powiadomienia zalene od globalnych ustawie */
	OFF = 1, /*!< powiadomienia wyczone */
	OWN = 2 /*!< indywidualne ustawienie powiadamiania */
};

/**
	Klasa reprezentujca kontakt (uytkownika na licie kontaktw).
	\class UserListElement
	\brief Klasa reprezentujca kontakt.
**/
class UserListElement : public QObject
{
	Q_OBJECT
	private:
		QString Group; /*!< Grupa do ktrej przynaley kontakt
						(jeli wicej ni jedna, to oddzielone s przecinkami). */
		QString FirstName; /*!< Imi kontaktu. */
		QString LastName; /*!< Nazwisko kontaktu. */
		QString NickName; /*!< Pseudonim kontaktu. */
		QString AltNick; /*!< Pseudonim kontaktu, ktry jest wywietlany na licie. */
		QString Mobile; /*!< Numer telefonu kontaktu. */
		QString Email; /*!< E-Mail kontaktu. */
		UinType Uin; /*!< UIN kontaktu. */
		UserStatus *Stat; /*!< Aktualny status kontaktu. */
		int MaxImageSize; /*!< Maksymalny rozmiar obrazka, jak moe przyj kontakt. */
		bool Anonymous; /*!< Informuje, czy kontakt jest anonimowy czy nie. */
		QHostAddress Ip; /*!< Adres IP kontaktu (jeli wykryty). */
		QString DnsName; /*!< Nazwa domenu kontaktu (jeli znaleziona). */
		short Port; /*!< Port kontaktu (jeli wykryty). */
		int Version; /*!< Wersja protokou uywanego przez kontakt. */
		bool Blocking; /*!< Informuje czy blokujemy kontakt, czy nie. */
		bool OfflineTo; /*!< Informuje czy mamy wczony tryb "niedostpny dla kontaktu" dla tego kontaktu. */
		bool Notify; /*!< Informuje czy mamy wczone powiadamianie o kontakcie. */
		
		NotifyType AliveSound; /*!< Przechowuje informacj o sposobie powiadamiania
								o zmianie statusu kontaku dwikiem. */
		QString OwnAliveSound; /*!< Jeli sposb powiadamiania o zmianie statusu kontaktu ma warto OWN,
								to ta zmienna przechowuje nazw pliku dwikowego do odtworzenia. */

		NotifyType MessageSound; /*!< Przechowuje informacj o sposobie powiadamiania
								o nowej wiadomoci od kontaktu dwikiem. */
		QString OwnMessageSound; /*!< Jeli sposb powiadamiania o nowej wiadomoci od kontaktu ma warto OWN,
								to ta zmienna przechowuje nazw pliku dwikowego do odtworzenia. */
		
		QString HomePhone; /*!< Numer telefonu domowego kontaktu. */

		UserList *Parent; /*!< Wskanik do listy kontaktw, na ktrej znajduje si ten kontakt. */
		friend class UserList;
	public slots:
		/**
			\fn void setGroup(const QString& group)
			Ustawia grup (lub grupy, oddzielone przecinkami) dla kontaktu.
			\param group grupa (grupy) do ktrej zostanie przydzielony kontakt.
		**/
		void setGroup(const QString& group);
		
		/**
			\fn void setFirstName(const QString &firstName)
			Ustawia imi dla kontaktu.
			\param firstName imi, ktre zostanie przydzielone kontaktowi.
		**/
		void setFirstName(const QString &firstName);
		
		/**
			\fn void setLastName(const QString &lastName)
			Ustawia nazwisko dla kontaktu.
			\param lastName nazwisko, ktre zostanie przydzielone kontaktowi.
		**/
		void setLastName(const QString &lastName);
		
		/**
			\fn void setNickName(const QString &nickName)
			Ustawia pseudonim dla kontaktu.
			\param nickName pseudonim, ktry zostanie przydzielony kontaktowi.
		**/
		void setNickName(const QString &nickName);
		
		/**
			\fn void setAltNick(const QString &altNick)
			Ustawia wywietlany pseudonim dla kontaktu.
			\param altNick wywietlany pseudonim, ktry zostanie przydzielony kontaktowi.
		**/
		void setAltNick(const QString &altNick);
		
		/**
			\fn void setMobile(const QString &mobile)
			Ustawia numer telefonu dla kontaktu.
			\param mobile numer telefonu, ktry zostanie przydzielony kontaktowi.
		**/
		void setMobile(const QString &mobile);
		
		/**
			\fn void setEmail(const QString &email)
			Ustawia adres E-Mail dla kontaktu.
			\param email adres, ktry zostanie przydzielony kontaktowi.
		**/
		void setEmail(const QString &email);
		
		/**
			\fn void setUin(const UinType &uin)
			Ustawia UIN dla kontaktu.
			\param uin numer UIN, ktry zostanie przydzielony kontaktowi.
		**/
		void setUin(const UinType &uin);
		
		/**
			\fn void setMaxImageSize(const int maxImageSize)
			Ustawia maksymalny rozmiar obrazka dla kontaktu.
			\param maxImageSize rozmiar obrazka w bajtach, ktry zostanie przydzielony kontaktowi.
		**/
		void setMaxImageSize(const int maxImageSize);
		
		/**
			\fn void setAnonymous(const bool anonymous)
			Ustawia stan anonimowoci kontaktu.
			\param anonymous warto logiczna informujca, e kontakt jest anonimowy, lub nie.
		**/
		void setAnonymous(const bool anonymous);
		
		/**
			\fn void setDnsName(const QString &dnsName)
			Ustawia nazw domeny dla kontaktu.
			\param dnsName nazwa domenu, ktra zostanie przydzielona kontaktowi.
		**/
		void setDnsName(const QString &dnsName);
		
		/**
			\fn void setPort(short port)
			Ustawia numer portu dla kontaktu.
			\param port numer portu, ktry zostanie przydzielony kontaktowi.
		**/
		void setPort(short port);
		
		/**
			\fn void setVersion(const int version)
			Ustawia wersj protokou dla kontaktu.
			\param version wersja protokou, ktra zostanie przydzielona kontaktowi.
		**/
		void setVersion(const int version);
		
		/**
			\fn void setBlocking(const bool blocking)
			Ustawia stan blokowania kontaktu.
			\param blocking warto logiczna informujca, czy kontakt ma by blokowany.
		**/
		void setBlocking(const bool blocking);
		
		/**
			\fn void setOfflineTo(const bool offlineTo)
			Ustawia stan niedostpnoci dla kontaktu.
			\param offlineTo warto logiczna informujca, czy uytkownik ma by "niedostpny dla kontaktu".
		**/
		void setOfflineTo(const bool offlineTo);
		
		/**
			\fn void setNotify(const bool notify)
			Ustawia stan powiadamiania o kontakcie.
			\param notify warto logiczna informujca, czy uytkownik ma by powiadamiany o kontakcie.
		**/
		void setNotify(const bool notify);

		/**
			\fn void setHomePhone(const QString &phone)
			Ustawia numer telefonu domowego dla kontaktu.
			\param phone numer telefonu, ktry zostanie przydzielony kontaktowi.
		**/
		void setHomePhone(const QString &phone);
		
		/**
			\fn void setAliveSound(NotifyType type, const QString &file="")
			Ustawia sposb powiadamiania dwikiem o zmianie statusu przez kontakt.
			\param type sposb powiadamiania.
			\arg \c GLOBAL powiadomienia zalene od globalnych ustawie.
			\arg \c OFF powiadomienia wyczone.
			\arg \c OWN indywidualne ustawienie powiadamiania/

			\param file plik dwikowy, wymagany dla indywidualnego ustawienia powiadomie.
		**/
		void setAliveSound(NotifyType type, const QString &file="");
		
		/**
			\fn void setMessageSound(NotifyType type, const QString &file="")
			Ustawia sposb powiadamiania dwikiem o nowej wiadomoci od kontaktu.
			\param type sposb powiadamiania.
			\arg \c GLOBAL powiadomienia zalene od globalnych ustawie.
			\arg \c OFF powiadomienia wyczone.
			\arg \c OWN indywidualne ustawienie powiadamiania/

			\param file plik dwikowy, wymagany dla indywidualnego ustawienia powiadomie.
		**/
		void setMessageSound(NotifyType type, const QString &file="");

	public:
		/**
			\fn UserListElement(UserList* parent)
			Konstruktor dodajcy kontakt do wskazanej listy kontaktw.
			\param parent wskanik listy kontaktw, do ktrej zostanie dodany kontakt.
		**/
		UserListElement(UserList* parent);

		/**
			\fn UserListElement(const UserListElement &copyMe)
			Konstruktor kopiujcy.
		**/
		UserListElement(const UserListElement &copyMe);

		/**
			\fn UserListElement()
			Domylny konstruktor tworzcy kontakt o nie wypenionych jeszcze danych.
		**/
		UserListElement();
		virtual ~UserListElement();

		/**
			\fn void refreshDnsName()
			Wywouje zapytanie o nazw domeny dla kontaktu oraz wypenia odpowiednie pole kontaktu,
			gdy domena zostanie odnaleziona.
		**/
		void refreshDnsName();

		/**
			\fn void operator = (const UserListElement &copyMe)
			Operator kopiujcy.
			\param copyMe obiekt klasy UserListElement ktry bdzie skopiowany.
		**/
		void operator = (const UserListElement &copyMe);

		/**
			\fn const QString &group() const
			Zwraca grup (lub wiele grup oddzielonych przecinkami) do ktrej naley kontakt.
		**/
		const QString &group() const;
		
		/**
			\fn const QString &firstName() const
			Zwraca imi kontaktu.
		**/
		const QString &firstName() const;
		
		/**
			\fn const QString &lastName() const
			Zwraca nazwisko kontaktu.
		**/
		const QString &lastName() const;
		
		/**
			\fn const QString &nickName() const
			Zwraca pseudonim kontaktu.
		**/
		const QString &nickName() const;
		
		/**
			\fn const QString &altNick() const
			Zwraca wywietlany pseudonim kontaktu.
		**/
		const QString &altNick() const;
		
		/**
			\fn const QString &mobile() const
			Zwraca numer telefonu kontaktu.
		**/
		const QString &mobile() const;
		
		/**
			\fn const QString &email() const
			Zwraca adres E-Mail kontaktu.
		**/
		const QString &email() const;
		
		/**
			\fn UinType uin() const
			Zwraca numer UIN kontaktu.
		**/
		UinType uin() const;
		
		/**
			\fn const QString &homePhone() const
			Zwraca numer telefonu domowego kontaktu.
		**/
		const QString &homePhone() const;
		
		/**
			\fn const QString &aliveSound(NotifyType &type) const
			Zwraca sposb powiadamiania dwikiem o zmianach statusu kontaktu.
		**/
		const QString &aliveSound(NotifyType &type) const;
		
		/**
			\fn const QString &messageSound(NotifyType &type) const
			Zwraca sposb powiadamiania dwikiem o wiadomociach od kontaktu.
		**/
		const QString &messageSound(NotifyType &type) const;

		/**
			\fn const UserStatus & status() const
			Zwraca aktualny status kontaktu.
		**/
		const UserStatus & status() const;
		
		/**
			\overload UserStatus & status()
		**/
		UserStatus & status();

		/**
			\fn int maxImageSize() const
			Zwraca maksymalny rozmiar obrazka dla kontaktu.
		**/
		int maxImageSize() const;
		
		/**
			\fn bool isAnonymous() const
			Zwraca informacj, czy kontakt jest anonimowy czy nie.
		**/
		bool isAnonymous() const;

		/**
			\fn const QHostAddress & ip() const
			Zwraca adres IP kontaktu (jeli wykryty).
		**/
		const QHostAddress & ip() const;
		
		/**
			\overload QHostAddress & ip()
		**/
		QHostAddress & ip();
		
		/**
			\fn const QString &dnsName() const
			Zwraca nazw domeny kontaktu (jeli znaleziona).
		**/
		const QString &dnsName() const;
		
		/**
			\fn short port() const
			Zwraca numer portu kontaktu (jeli znaleziony).
		**/
		short port() const;
		
		/**
			\fn int version() const
			Zwraca wersj protokou, jakiej uywa kontakt.
		**/
		int version() const;
		
		/**
			\fn bool blocking() const
			Zwraca informacj, czy kontakt jest blokowany czy nie.
		**/
		bool blocking() const;
		
		/**
			\fn bool offlineTo() const
			Zwraca informacj, czy uytkownik jest "niedostpny dla kontaktu" czy nie.
		**/
		bool offlineTo() const;
		
		/**
			\fn bool notify() const
			Zwraca informacj, czy uytkownik jest powiadamiany o kontakcie czy nie.
		**/
		bool notify() const;
};

/**
	Indeksami s altnicki (wywietlane pseudonimy) konwertowane tak,
	aby wszystkie litery byy mae.
	Dziki temu nie mog istnie dwa kontakty
	o tych samych pseudonimach wywietlanych nawet, jeli
	rni si wielkociami liter.
	Pole Altnick w UserListElement zawiera natomiast
	oryginalny pseudonim wywietlany.
	
	\warning Uywanie metod klasy QMap nie jest zalecane,
	trzeba wtedy pamita, e jako klucza uywamy pseudonimw wywietlanych
	odpowiednio skonwertowanych, e zmiany powinny generowa
	odpowiednie sygnay, itd. Duo lepiej uywa specjalnie
	przygotowanych metod klasy UserList.
	\class UserList
	\brief Klasa reprezentujca list kontaktw.
	
	TODO: Zgodnie z tym co powyej z QMap powinnimy dziedziczy
	prywatnie albo niech to bdzie pole wewntrz klasy. W tej
	chwili nie chc ama api freeze w 0.4.0.
**/
class UserList : public QObject, public QMap<QString,UserListElement>
{
	Q_OBJECT

	protected:
		friend class UserListElement;

	public:
		/**
			\fn UserList()
			Konstruktor standardowy.
		**/
		UserList();

		/**
			\fn UserList(const UserList& source)
			Konstruktor kopiujcy.
			\param source lista kontaktw, ktr naley skopiowa.
		**/
		UserList(const UserList& source);
		~UserList();

		/**
			\fn UserList& operator=(const UserList& userlist)
			Operator kopiujcy.
			\param userlist lista kontaktw, ktr naley skopiowa.
		**/
		UserList& operator=(const UserList& userlist);

		/**
			\fn UserListElement& byUin(UinType uin)
			Wyszukuje kontakt z numerem \a uin.
			\param uin numer UIN wg. ktrego nastpuje wyszukiwanie.
			\return referencj do obiektu reprezentujcego kontakt o podanym UIN'ie.
		**/
		UserListElement& byUin(UinType uin);
		
		/**
			\fn UserListElement& byNick(const QString& nickname);
			Wyszukuje kontakt po pseudonimie. Mae i
			due litery nie maj znaczenia.
			\param nickname pseudonim wg. ktrego nastpuje wyszukiwanie.
			\return referencj do obiektu reprezentujcego kontakt o podanym pseudonimie.
		**/
		UserListElement& byNick(const QString& nickname);

		/**
			\fn UserListElement& byAltNick(const QString& altnick)
			Wyszukuje kontakt po wywietlanym pseudonimie. Mae i
			due litery nie maj znaczenia.
			\param altnick wywietlany pseudonim wg. ktrego nastpuje wyszukiwanie.
			\return referencj do obiektu reprezentujcego kontakt o podanym pseudonimie wywietlanym.
		**/
		UserListElement& byAltNick(const QString& altnick);

		/**
			\fn UserListElement byUinValue(UinType uin)
			Wyszukuje kontakt po numerze \a uin.
			\param uin UIN wg. ktrego nastpuje wyszukiwanie.
			\return obiekt reprezentujcy kontakt z podanym UIN'em. Jezeli nie znajdzie, zwracany jest UserListElement zawierajcy tylko UIN i pseudonim wywietlany (identyczny z numerem UIN).
		**/
		UserListElement byUinValue(UinType uin);

		/**
			\fn bool containsUin(UinType uin) const
			Sprawdza czy istnieje kontakt o danym numerze uin.
			\param uin sprawdzany UIN.
		**/
		bool containsUin(UinType uin) const;

		/**
			\fn bool containsAltNick(const QString& altnick) const;
			Sprawdza czy istnieje kontakt o danym pseudonimie wywietlanym.
			Mae i due litery nie maj znaczenia.
			\param altnick Sprawdzany pseudonim wywietlany.
		**/
		bool containsAltNick(const QString& altnick) const;

	public slots:
		/**
			\fn void addUser(const UserListElement& ule)
			Dodaje do listy kontakt o podanych danych.
			Generuje nastpnie sygnay userDataChanged()
			i modified().
			\param ule obiekt reprezentujcy kontakt, ktry chcemy doda.
		**/
		void addUser(const UserListElement& ule);

		/**
			\fn void addAnonymous(UinType uin)
			Dodaje do listy "anonimowy" kontakt o podanym
			numerze UIN. Numer ten wykorzystywany jest jako
			pseudonim wywietlany. Wywouje metod addUser(), wic
			generowane s sygnay userDataChanged() i modified().
			\param uin UIN kontaktu, ktry chcemy doda.
		**/
		void addAnonymous(UinType uin);

		/**
			\fn void removeUser(const QString& altnick)
			Usuwa kontakt o podanym pseudonimie wywietlanym. Mae i
			due litery nie maj znaczenia.
			Tu przed usuniciem kontaktu z listy generuje
			sygna userDataChanged(). Po usuniciu generowany
			jest sygna modified().
			\param altnick pseudonim wywietlany kontaktu, ktry chcemy usun.
		**/
		void removeUser(const QString& altnick);

		/**
			\fn void changeUserInfo(const QString& old_altnick, const UserListElement& new_data)
			Zmienia dane kontaktu o podanym pseudonimie wywietlanym. Mae i
			due litery w zmiennej \a old_altnick nie maj znaczenia.
			Po wykonaniu zmian wymusza ich uwzgldnienie przez
			wszystkie obiekty klasy UserBox wywoujc metod
			UserBox::all_renameUser() i UserBox::all_refresh().
			Na koniec generuje sygnay userDataChanged() i modified().
			\param old_altnick dotychczasowy pseudonim wywietlany kontaktu.
			\param new_data obiekt reprezentujcy nowe dane kontaktu.
		**/
		void changeUserInfo(const QString& old_altnick, const UserListElement& new_data);

		/**
			\fn bool writeToFile(QString filename = "")
			Zapisuje list kontaktw do pliku o podanej nazwie
			(wraz ze ciek). Domylnie jest to plik "userlist"
			w katalogu z ustawieniami.
			Utrzymywana jest kompatybilno z programem EKG,
			z tej przyczyny dodatkowe informacje jak to czy
			blokujemy uytkownika, czy chcemy by powiadamiani
			o zmianach jego statusu itp. s zapisywane do
			oddzielnego pliku "userattribs" w katalogu z
			ustawieniami.
			Kontakty "anonimowe" s pomijane.
			\param filename plik do ktrego zostanie zapisana lista.
			\return FALSE, gdy wystpiy bdy, lub TRUE w przeciwnym wypadku.
			
			TODO: To gdzie zapisujemy dodatkowe atrybutami te
			powinno by chyba podawane jako argument? Inaczej
			jeli chcemy zapisa list gdzie indziej to nadpisz
			nam si atrybuty :/
		**/
		bool writeToFile(QString filename = "");

		/**
			\fn bool readFromFile()
			Wczytuje list kontaktw z plikw "userlist"
			i "userattribs" z katalogu z ustawieniami.
			Metoda ta jest przeznaczona do przywracania
			danych zapisanych za pomoc writeToFile().
			\return FALSE, gdy wystpiy bdy, lub TRUE w przeciwnym wypadku.
		**/
		bool readFromFile();

		/**
			\fn void merge(const UserList &userlist)
			Scala list kontaktw z podan list kontaktw \a userlist.
			\param userlist lista kontaktw z ktr naley scali
		**/
		void merge(const UserList &userlist);

	signals:
		/**
			\fn void modified()
			Sygna generowany po zakoczeniu zmian w licie
			kontaktw.
		**/
		void modified();

		/**
			\fn void dnsNameReady(UinType)
			Sygna generowany, gdy adres IP kontaktu o danym UIN'ie zostanie przetumaczony
			na nazw domeny.
			\param uin UIN kontaktu, dla ktrego przygotowano nazw domeny.
		**/
		void dnsNameReady(UinType uin);

		/**
			\fn void userDataChanged(const UserListElement* const oldData, const UserListElement* const newData, bool massively = false)
			Sygna jest generowany jeli cokolwiek zmienio
			si w danych ktrego z kontaktw na licie
			(wtedy przekazywane s argumenty \a oldData i \a newData).
			Zmiana moe nastpi przez bezporednie uycie ktrej
			z metod klasy UserListElement.
			Sygna przekazuje rwnie informacj o tym, e nowy
			kontakt pojawi si na licie,
			lub e zosta z listy usunity.
			Jeeli ta zmiana jest czci czego wikszego, to \a massively == true
			i emitowany jest sygna UserList::allNewContacts.
			
			\param oldData obiekt reprezentujcy stare dane kontaktu. Jeli ten parametr ma warto NULL,
				to znaczy, e kontakt wanie zosta dodany do listy.
			\param newData obiekt reprezentujcy nowe dane kontaktu. Jeli ten parametr ma warto NULL,
				to znaczy, e kontakt zosta wanie usunity z listy.
		**/
		void userDataChanged(const UserListElement* const oldData, const UserListElement* const newData, bool massively = false);

		/**
			\fn void allNewContacts(const UserList &ulist)
			Sygna jest emitowany, kiedy powstaje zupenie nowa lista kontaktw.
		**/
		void allNewContacts(const UserList &ulist);
};

extern UserList userlist;

#endif

